import { FlexStyle } from '../layout/index.js';
import { colors } from './utils/color.js';
export type StylePropertyConfig<T = any> = {
    setNative: (style: Style, key: string, value: T, config: StylePropertyConfig<T>) => void;
    getNative?: (style: Style) => any;
    shouldLayout?: boolean;
    hasChanged?: (oldValue: T, newValue: T) => boolean;
    converter?: {
        toNative?: (key: string, value: T) => any;
        fromNative?: (key: string, value: any) => T;
    };
    defaultValue?: T;
};
/**
 * A function that decorates properties on the style object.
 */
export declare function style<T>(config: StylePropertyConfig<T>): (_target: Style, property: string) => void;
export declare function flex(): (_target: Style, property: string) => void;
export interface ViewStyle extends FlexStyle {
    backgroundColor?: `${Lowercase<keyof typeof colors>}` | ({} & string);
    borderColor?: `${Lowercase<keyof typeof colors>}` | ({} & string);
    borderRadius?: number;
    opacity?: number;
}
export interface TextStyle extends ViewStyle {
    color?: `${Lowercase<keyof typeof colors>}` | ({} & string);
    fontSize?: string | number;
    fontFamily?: string;
    fontStyle?: string;
}
export interface CombinedStyle extends ViewStyle, TextStyle {
}
export declare class Style extends Map {
    node: any;
    /**
     * Install a new style property.
     */
    static install: typeof style;
    pendingSetNative: Map<string, () => void>;
    constructor(node: any);
    set<Key extends keyof CombinedStyle>(key: Key, value: CombinedStyle[Key]): this;
    get<Key extends keyof CombinedStyle>(key: Key): CombinedStyle[Key] | undefined;
    clear(): void;
    alignContent: FlexStyle['alignContent'];
    alignItems: FlexStyle['alignItems'];
    alignSelf: FlexStyle['alignSelf'];
    aspectRatio: FlexStyle['aspectRatio'];
    borderWidth: FlexStyle['borderWidth'];
    direction: FlexStyle['direction'];
    display: FlexStyle['display'];
    flex: FlexStyle['flex'];
    flexBasis: FlexStyle['flexBasis'];
    flexDirection: FlexStyle['flexDirection'];
    flexGrow: FlexStyle['flexGrow'];
    flexShrink: FlexStyle['flexShrink'];
    flexWrap: FlexStyle['flexWrap'];
    width: FlexStyle['width'];
    height: FlexStyle['height'];
    justifyContent: FlexStyle['justifyContent'];
    gap: FlexStyle['gap'];
    rowGap: FlexStyle['rowGap'];
    columnGap: FlexStyle['columnGap'];
    marginLeft: FlexStyle['marginLeft'];
    marginRight: FlexStyle['marginRight'];
    marginTop: FlexStyle['marginTop'];
    marginBottom: FlexStyle['marginBottom'];
    maxHeight: FlexStyle['maxHeight'];
    maxWidth: FlexStyle['maxWidth'];
    minHeight: FlexStyle['minHeight'];
    minWidth: FlexStyle['minWidth'];
    overflow: FlexStyle['overflow'];
    paddingLeft: FlexStyle['paddingLeft'];
    paddingRight: FlexStyle['paddingRight'];
    paddingTop: FlexStyle['paddingTop'];
    paddingBottom: FlexStyle['paddingBottom'];
    padding: FlexStyle['padding'];
    top: FlexStyle['top'];
    left: FlexStyle['left'];
    right: FlexStyle['right'];
    bottom: FlexStyle['bottom'];
    end: FlexStyle['end'];
    start: FlexStyle['start'];
    marginBlock: FlexStyle['marginBlock'];
    marginBlockEnd: FlexStyle['marginBlockEnd'];
    marginBlockStart: FlexStyle['marginBlockStart'];
    paddingBlock: FlexStyle['paddingBlock'];
    paddingBlockEnd: FlexStyle['paddingBlockEnd'];
    paddingBlockStart: FlexStyle['paddingBlockStart'];
    paddingHorizontal: FlexStyle['paddingHorizontal'];
    paddingVertical: FlexStyle['paddingVertical'];
    marginVertical: FlexStyle['marginVertical'];
    marginHorizontal: FlexStyle['marginHorizontal'];
    marginStart: FlexStyle['marginStart'];
    marginEnd: FlexStyle['marginEnd'];
    paddingStart: FlexStyle['paddingStart'];
    backgroundColor: ViewStyle['backgroundColor'];
    zIndex: ViewStyle['zIndex'];
    fontSize: TextStyle['fontSize'];
    fontStyle: TextStyle['fontStyle'];
    color: TextStyle['color'];
    borderRadius: ViewStyle['borderRadius'];
    borderColor: ViewStyle['borderColor'];
    opacity: ViewStyle['opacity'];
}
