import { MeasureMode } from 'yoga-layout/load';
import { ViewBase } from '../view/view-base.js';
export class TextBase extends ViewBase {
    get isLeaf() {
        // Text is always a leaf node.
        return true;
    }
    updateTextContent() {
        console.warn('TextBase.updateTextContent() is not implemented ');
    }
    onMeasureFunction(width, widthMode, height, heightMode) {
        if (this.nativeView?.sizeThatFits) {
            return super.onMeasureFunction(width, widthMode, height, heightMode);
        }
        const constrainedWidth = widthMode === MeasureMode.Undefined ? Number.MAX_VALUE : width;
        const constrainedHeight = heightMode === MeasureMode.Undefined ? Number.MAX_VALUE : height;
        const text = this.nativeView;
        const size = !text
            ? { width: 0, height: 0 }
            : NSAttributedString.alloc()
                .initWithStringAttributes(text.string || '', {
                [NSFontAttributeName]: text.font,
                [NSKernAttributeName]: 1,
            })
                .boundingRectWithSizeOptionsContext({
                width: constrainedWidth,
                height: constrainedHeight,
            }, NSStringDrawingOptions.UsesFontLeading | NSStringDrawingOptions.UsesLineFragmentOrigin, null).size;
        return {
            width: this.measure(constrainedWidth, Math.ceil(size.width), widthMode),
            height: this.measure(constrainedHeight, Math.ceil(size.height), heightMode),
        };
    }
    connectedCallback() {
        super.connectedCallback();
        this.updateTextContent();
    }
}
//# sourceMappingURL=text-base.js.map