import { MeasureMode } from 'yoga-layout/load';
import { YogaNode, YogaNodeLayout } from '../../layout/index.js';
import { CombinedStyle, Style } from '../../style/index.js';
import { type NativePropertyConfig } from '../decorators/native.js';
export declare class ViewBase extends HTMLElement {
    static register(): void;
    parentNode: ViewBase | null;
    shouldAttachToParentNativeView: boolean;
    pauseLayoutUpdates?: boolean;
    viewController?: NSViewController;
    supportedNodeTypes?: number[];
    pendingSetNative: Map<string, () => void>;
    isRoot: boolean;
    nativeView?: any;
    _styleInternal?: Style;
    get styleInternal(): Style;
    _yogaNode?: YogaNode;
    _isDirty?: boolean;
    get yogaNode(): YogaNode;
    _rootView?: ViewBase;
    get rootYogaNode(): YogaNode;
    initNativeView(): any | undefined;
    prepareNativeView(_nativeView: any): void;
    disposeNativeView(): void;
    set style(style: CombinedStyle);
    get style(): Style;
    applyLayout(parentLayout?: YogaNodeLayout): void;
    applyLayoutToChildern(layout?: YogaNodeLayout): void;
    get isLeaf(): boolean;
    _isIncludedInLayout: boolean;
    _isEnabled: boolean;
    get isEnabled(): boolean;
    get isIncludedInLayout(): boolean;
    insertBefore<T extends Node>(node: T, child: Node | null): T;
    private setRootView;
    private _addYogaChild;
    onDocumentFragmentAdded(): void;
    /**
     * The onMeasureFunction method is responsible for determining the size
     * of a view by considering the provided width and height constraints.
     *  It uses the sizeThatFits method of the native view (if available)
     * to calculate the optimal size based on these constraints.
     * If the native view is not present, it simply returns the provided
     * dimensions.
     */
    onMeasureFunction(width: number, widthMode: MeasureMode, height: number, heightMode: MeasureMode): {
        width: number;
        height: number;
    };
    measure(constrainedSize: number, measuredSize: number, measureMode: MeasureMode): number;
    removeChild<T extends Node>(child: T): T;
    addNativeChild(child: any): void;
    removeNativeChild(child: any): void;
    connectedCallback(): void;
    disconnectedCallback(): void;
    setAttributeNS(_namespace: string | null, qualifiedName: string, value: string): void;
    getAttributeNS(_namespace: string | null, qualifiedName: string): any;
    setNativeProperties(): void;
    propertyChangedCallback(_property: string, _value: any, _oldValue: any): void;
    applySafeAreaPaddings(): void;
    _safeAreaPadding: boolean;
    enableSafeAreaPaddings: boolean;
    setPaddingTop(key: string, value: any, config: NativePropertyConfig): void;
}
