var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
import { MeasureMode } from 'yoga-layout/load';
import { Layout } from '../../layout/index.js';
import { Style } from '../../style/index.js';
import { native } from '../decorators/native.js';
import { overrides } from '../decorators/overrides.js';
export class ViewBase extends HTMLElement {
    constructor() {
        super(...arguments);
        this.shouldAttachToParentNativeView = true;
        this.pendingSetNative = new Map();
        this.isRoot = false;
        this._rootView = undefined;
        this._isIncludedInLayout = true;
        this._isEnabled = true;
        this._safeAreaPadding = false;
    }
    static register() {
        //@ts-ignore
        if (this._register) {
            //@ts-ignore
            this._register();
        }
        else {
            console.warn(`View ${this.name} is missing the @view decorator.`);
        }
    }
    get styleInternal() {
        if (!this._styleInternal) {
            return (this._styleInternal = new Style(this));
        }
        return this._styleInternal;
    }
    get yogaNode() {
        if (!this._yogaNode) {
            this._yogaNode = Layout.createYogaNode();
            if (this.isLeaf) {
                this._yogaNode.setMeasureFunc(this.onMeasureFunction.bind(this));
            }
        }
        return this._yogaNode;
    }
    get rootYogaNode() {
        if (!this._rootView)
            return this.yogaNode;
        return this.isRoot ? this.yogaNode : this._rootView?.yogaNode;
    }
    initNativeView() {
        return undefined;
    }
    prepareNativeView(_nativeView) { }
    disposeNativeView() {
        this.nativeView = undefined;
    }
    //@ts-ignore
    set style(style) {
        if (!style) {
            this.styleInternal.clear();
        }
        for (const key in style) {
            //@ts-ignore
            this.styleInternal[key] = style[key];
        }
    }
    //@ts-ignore
    get style() {
        return this.styleInternal;
    }
    applyLayout(parentLayout) {
        if (!this.isIncludedInLayout)
            return;
        const layout = this.yogaNode?.getComputedLayout();
        if (layout && this.nativeView) {
            const parentHeight = parentLayout?.height || 0;
            const width = isNaN(layout.width) ? 0 : layout.width;
            const height = isNaN(layout.height) ? 0 : layout.height;
            this.nativeView.translatesAutoresizingMaskIntoConstraints = false;
            this.nativeView.frame = {
                origin: {
                    x: layout.left,
                    // Reverse the origin so that view's are rendered from
                    // the top left instead of default bottom right.
                    y: parentLayout ? Math.max(parentHeight - layout.top - height, 0) : layout.top,
                },
                size: {
                    width,
                    height,
                },
            };
            this.applyLayoutToChildern(layout);
        }
    }
    applyLayoutToChildern(layout) {
        let child = this.firstChild;
        while (child) {
            if (child.nodeType === 1) {
                child.applyLayout?.(layout);
            }
            child = child.nextSibling;
        }
    }
    get isLeaf() {
        return false;
    }
    get isEnabled() {
        return this._isEnabled;
    }
    get isIncludedInLayout() {
        return this._isIncludedInLayout && this.isEnabled;
    }
    insertBefore(node, child) {
        if (this.supportedNodeTypes && !this.supportedNodeTypes.includes(node.nodeType)) {
            return node;
        }
        super.insertBefore(node, child);
        if (node.nodeType === 1 && this.isEnabled && !this.isLeaf && node.isEnabled && !node.isRoot) {
            this._addYogaChild(node, !!child);
        }
        this.setRootView(node);
        node.connectedCallback?.();
        return node;
    }
    setRootView(node) {
        node._rootView = node.isRoot ? node : this._rootView || this;
        let child = node.firstChild;
        while (child) {
            if (child.nodeType == Node.ELEMENT_NODE) {
                this.setRootView(child);
            }
            child = child.nextSibling;
        }
        if (node.isLeaf) {
            node.yogaNode.markDirty();
        }
    }
    _addYogaChild(node, hasRef) {
        if (hasRef) {
            // Remove all childern from yogaNode
            let child = this.firstChild;
            while (child) {
                if (child.isEnabled && child.yogaNode) {
                    this.yogaNode.removeChild(child.yogaNode);
                }
                child = child.nextSibling;
            }
            // Add all childern to yogaNode
            child = this.firstChild;
            while (child) {
                if (child.isEnabled && child.yogaNode) {
                    this.yogaNode.insertChild(child.yogaNode, this.yogaNode.getChildCount());
                }
                child = child.nextSibling;
            }
        }
        else if (node) {
            this.yogaNode.insertChild(node.yogaNode, this.yogaNode.getChildCount());
        }
    }
    onDocumentFragmentAdded() {
        // Reinsert all yoga childern in correct order after a document fragment is added
        this._addYogaChild(undefined, true);
    }
    /**
     * The onMeasureFunction method is responsible for determining the size
     * of a view by considering the provided width and height constraints.
     *  It uses the sizeThatFits method of the native view (if available)
     * to calculate the optimal size based on these constraints.
     * If the native view is not present, it simply returns the provided
     * dimensions.
     */
    onMeasureFunction(width, widthMode, height, heightMode) {
        if (this.nativeView?.sizeThatFits || this.nativeView?.sizeToFit) {
            const constrainedWidth = widthMode === MeasureMode.Undefined ? Number.MAX_VALUE : width;
            const constrainedHeight = heightMode === MeasureMode.Undefined ? Number.MAX_VALUE : height;
            const fittingSize = this.nativeView?.fittingSize;
            let size = this.nativeView?.sizeThatFits?.({
                width: constrainedWidth,
                height: constrainedHeight,
            }) ||
                fittingSize || { width: 0, height: 0 };
            size = {
                width: Math.max(size.width, fittingSize.width),
                height: Math.max(size.height, fittingSize.height),
            };
            return {
                width: this.measure(constrainedWidth, size.width, widthMode),
                height: this.measure(constrainedHeight, size.height, heightMode),
            };
        }
        return { width: width, height: height };
    }
    measure(constrainedSize, measuredSize, measureMode) {
        let result;
        if (measureMode === MeasureMode.Exactly) {
            result = constrainedSize;
        }
        else if (measureMode === MeasureMode.AtMost) {
            result = Math.min(constrainedSize, measuredSize);
        }
        else {
            result = measuredSize;
        }
        return result;
    }
    removeChild(child) {
        super.removeChild(child);
        if (child.nodeType == 1) {
            child.shouldAttachToParentNativeView && this.removeNativeChild(child);
            child.disconnectedCallback?.();
        }
        return child;
    }
    addNativeChild(child) {
        this.nativeView?.addSubview(child.nativeView);
    }
    removeNativeChild(child) {
        child.nativeView?.removeFromSuperview();
    }
    connectedCallback() {
        if (this.parentNode && !this.parentNode?.isConnected) {
            return;
        }
        //@ts-ignore
        this.isConnected = true;
        if (!this.nativeView) {
            const nativeView = this.initNativeView();
            this.prepareNativeView(nativeView);
        }
        let parentHasUpdatesPaused = false;
        this.pauseLayoutUpdates = true;
        let current = this.parentNode;
        // Find a parent that has updates paused.
        // If such parent is found, we won't call layout updates
        // from this node as we want to do a single layout pass once
        // all nodes are loaded.
        if (!this.isRoot) {
            while (current) {
                if (current.pauseLayoutUpdates) {
                    parentHasUpdatesPaused = true;
                    break;
                }
                // Break where a root is found.
                current = current.isRoot ? null : current.parentNode;
            }
        }
        for (let i = 0; i < this.children.length; i++) {
            const child = this.children[i];
            if (child.nodeType === 1) {
                child.connectedCallback?.();
            }
        }
        this.setNativeProperties();
        /**
         * Some views might not want to attach to the parent native view.
         * For example, the window element.
         */
        if (this.parentNode && this.shouldAttachToParentNativeView) {
            this.viewController = this.viewController || this.parentNode.viewController;
            this.parentNode.addNativeChild(this);
        }
        if (this.pauseLayoutUpdates) {
            this.pauseLayoutUpdates = false;
            // If parentHasUpdatesPaused is true, some parent has paused layout updates, so we don't need to layout now.
            // the parent will do it eventually.
            if (!parentHasUpdatesPaused) {
                Layout.computeAndLayout(this);
            }
        }
    }
    disconnectedCallback() {
        //@ts-ignore
        this.isConnected = false;
        let childNode = this.firstChild;
        while (childNode) {
            if (childNode.nodeType === 1) {
                //@ts-ignore
                childNode.disconnectedCallback();
            }
            childNode = childNode.nextSibling;
        }
        if (this.isEnabled && !this.isRoot) {
            this.yogaNode.getParent()?.removeChild(this.yogaNode);
        }
        if (this.yogaNode) {
            this.yogaNode.free();
            this._yogaNode = undefined;
        }
        Layout.computeAndLayout(this._rootView);
        this._rootView = undefined;
        Promise.resolve().then(() => {
            if (!this.parentNode) {
                this.disposeNativeView();
            }
        });
    }
    setAttributeNS(_namespace, qualifiedName, value) {
        //@ts-ignore
        this[qualifiedName] = value;
    }
    getAttributeNS(_namespace, qualifiedName) {
        //@ts-ignore
        return this[qualifiedName];
    }
    setNativeProperties() {
        if (this.nativeView) {
            this.pendingSetNative.forEach((value) => value());
            this.style.pendingSetNative.forEach((value) => value());
        }
    }
    propertyChangedCallback(_property, _value, _oldValue) { }
    applySafeAreaPaddings() {
        if (this._safeAreaPadding) {
            let topSafeAreaPadding = 0;
            let window = this.nativeView.window;
            // Find nearest window element
            if (this.nodeName !== 'WINDOW' && !window) {
                let currentNode = this.parentNode;
                while (window == null && currentNode !== null) {
                    if (currentNode.nodeName === 'WINDOW') {
                        window = currentNode.nativeView;
                        break;
                    }
                    currentNode = currentNode.parentNode;
                }
            }
            if (this.nodeName === 'WINDOW') {
                window = this.nativeView;
            }
            if (window) {
                const frame = window.frame;
                const contentRect = window.contentLayoutRect;
                topSafeAreaPadding = frame.size.height - contentRect.size.height;
            }
            Layout.Setters.paddingTop(this.yogaNode, topSafeAreaPadding);
        }
        else {
            Layout.Setters.paddingTop(this.yogaNode, parseInt(this.getAttribute('paddingTop') || ''));
        }
    }
    setPaddingTop(key, value, config) {
        if (this._safeAreaPadding) {
            this.applySafeAreaPaddings();
            return;
        }
        config.setNative?.(this, key, value, config);
    }
}
__decorate([
    native({
        setNative(view, _key, value) {
            view._safeAreaPadding = value;
            view.applySafeAreaPaddings();
        },
        shouldLayout: true,
    }),
    __metadata("design:type", Boolean)
], ViewBase.prototype, "enableSafeAreaPaddings", void 0);
__decorate([
    overrides('paddingTop'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, Object, Object]),
    __metadata("design:returntype", void 0)
], ViewBase.prototype, "setPaddingTop", null);
//# sourceMappingURL=view-base.js.map